@@ -30,7 +30,7 @@ API_VERSION = 'v0'
 ISO_DATE_FORMAT = '%Y-%m-%dT%H:%M:%S'
 
 #: Default timeout option.
-TIMEOUT_SECS = 5
+TIMEOUT_SECS = 55
 
 #: Default retry options.
 RETRY = {
@@ -40,7 +40,7 @@ RETRY = {
     'redirect': 5,
     'method_whitelist': frozenset([
         'HEAD', 'GET', 'PUT', 'DELETE', 'OPTIONS', 'TRACE', 'POST']),
-    'status_forcelist': frozenset([500, 502, 503, 504]),
+    'status_forcelist': frozenset([502, 503, 504]),
     'backoff_factor': 1,
 }
 
@@ -51,9 +51,9 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
     :param client_id: The client id for accessing the API.
     :param client_secret: The client secret for accessing the API.
     :param cluster: The cluster endpoint for accessing the API.
-    :param user_api_url: Endpiont URL for the user API service.
+    :param user_api_url: Endpoint URL for the user API service.
     :param topic_api_url: Endpoint URL for the topic API service.
-    :param bulk_api_url: Endpoint URL fo the topic API service.
+    :param bulk_api_url: Endpoint URL for the topic API service.
     :param tenant: Tenant domain.
     :param version: API version to use, defaults to 'v0'.
     :param requests: `requests` instance for HTTP calls.
@@ -62,19 +62,19 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
     :param retry_connect: How many connection-related errors to retry on.
     :param retry_read: How many times to retry on read errors.
     :param retry_redirect: How many redirects to perform.
-    :param retry_method_whitelist: Set of uppercased HTTP method verbs that we
+    :param retry_method_whitelist: Set of uppercase HTTP method verbs that we
         should retry on.
     :param retry_status_forcelist: A set of integer HTTP status codes that we
         should force a retry on.
     :param retry_backoff_factor: A backoff factor to apply between attempts.
-    :param timeout_secs: How many seconds to wait for data before giving up.
+    :param timeout_secs: How many seconds to wait for data before giving up (default 55).
 
     Example::
 
         >>> from squirro_client import SquirroClient
         >>> client = SquirroClient('client_id', 'client_secret')
         >>> client = SquirroClient(None, None, cluster='http://squirro.example.com')
-        >>> client = SquirroClient(None, None, cluster='http://squiror.example.com', retry_total=10)
+        >>> client = SquirroClient(None, None, cluster='http://squirro.example.com', retry_total=10)
     """
 
     def __init__(self, client_id, client_secret, **kwargs):
@@ -153,7 +153,8 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
 
     def authenticate(self, tenant=None, access_token=None, refresh_token=None,
                      auth_service=None, auth_user=None,
-                     username=None, password=None):
+                     username=None, password=None,
+                     user_information=None):
         """Authenticate with the Squirro platform by either using access and
         refresh tokens, username and password, or external service name and
         user identifier.
@@ -165,6 +166,8 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
         :param auth_user: External authentication user identifier.
         :param username: User name.
         :param password: User password.
+        :param user_information: Additional information about the user for
+            internal use.
 
         :raises: :class:`squirro_client.exceptions.AuthenticationError` if authentication fails.
 
@@ -188,8 +191,10 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
             base['client_secret'] = self.client_secret
         if not self.tenant is None:
             base['tenant'] = self.tenant
+        if not user_information is None:
+            base['user_information'] = json.dumps(user_information)
 
-        # walk through all possible authentication scenearios
+        # walk through all possible authentication scenarios
         # - access token
         # - refresh token
         # - external id
@@ -233,7 +238,7 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
         platform.
         """
 
-        # create endpoint url to fecth new tokens
+        # create endpoint url to fetch new tokens
         url = '%(endpoint)s/oauth2/token' % ({'endpoint': self.user_api_url})
 
         session = self._new_session()
@@ -271,6 +276,7 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
             self.user_id = res.get('user_id')
         self.user_permissions = res.get('role_permissions', [])
         self.token_project_permissions = res.get('project_permissions', None)
+        self.user_information = res.get('user_information', {})
 
         # all is good and we store the new token
         self._on_token_refresh(
@@ -351,10 +357,10 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
 
         return res
 
-    def _refresh_tokens(self):
+    def _refresh_tokens(self, **kwargs):
         """Refresh the access token by using the refresh token."""
 
-        # return value is boolean
+        # return value is whether we managed to receive a new token
         ret = False
 
         # first we make sure that we have a refresh token
@@ -364,22 +370,42 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
 
         # login via the refresh token
         try:
-            self.authenticate(refresh_token=self.refresh_token)
-            ret = True
+            # Check that we have actually received a new access token.
+            # On multi-node systems we may not get a new one because of
+            # time-drift among the servers. Hence the server that we are about
+            # to ask to refresh the token may not consider the access token
+            # expired in contrast to a different server that has determined
+            # the access token has expired and whose response has triggered
+            # this `_refresh_tokens` request. Perform a couple of retries
+            # in a "best effort" manner until we receive a new access token.
+            previous_access_token = self.access_token
+
+            for try_count in range(3):
+                self.authenticate(refresh_token=self.refresh_token, **kwargs)
+
+                if self.access_token != previous_access_token:
+                    ret = True
+                    break
+
+                log.info('Access token %r not refreshed on behalf of ' + \
+                         'refresh token %r. Retry number %d',
+                         self.access_token, self.refresh_token, try_count)
+
         except Exception as ex:
             log.debug('unable to refresh access token: %r', ex)
 
+
         return ret
 
-    def _process_response(self, res, exected_status_codes=None):
-        """Return the data from the resonse if the status code matches to what
+    def _process_response(self, res, expected_status_codes=None):
+        """Return the data from the response if the status code matches to what
         is expected. If that is not the case the appropriate exception is
         being raised.
         """
 
         # work around mutable default arguments
-        if exected_status_codes is None:
-            exected_status_codes = [200]
+        if expected_status_codes is None:
+            expected_status_codes = [200]
 
         data = {}
         try:
@@ -390,7 +416,7 @@ class SquirroClient(UserApiMixin, TopicApiMixin, BulkApiMixin):
             raise TransportError(None, ex)
 
         # if the status code matches to what is expected we return the data
-        if res.status_code in exected_status_codes:
+        if res.status_code in expected_status_codes:
             return data
 
         # raise the appropriate exception
