@@ -4,11 +4,16 @@ import json
 class BulkApiMixin(object):
     """Mixin class which encapsulates the functionality of the bulk API
     service.
+
+    All the methods of this class are made available in the
+    [[SquirroClient]] class.
+
+    {TOC}
     """
 
     def new_shim(self, config):
         """Register a new shim with the bulk API. If the corresponding
-        configuration has already been registered the correspondig identifier
+        configuration has already been registered the corresponding identifier
         is returned instead.
 
         :param config: Shim configuration dictionary.
@@ -247,7 +252,8 @@ class BulkApiMixin(object):
 
         :param project_id: Project identifier.
         :param object_ext_ids: List of external object identifiers (optional).
-        :returns: A list of dicts which contain information about the object.
+        :returns: A list of dictionaries which contain information about the
+            object.
 
         Example::
 
@@ -278,7 +284,8 @@ class BulkApiMixin(object):
 
         :param shim_id: Shim identifier.
         :param object_ext_ids: List of external object identifiers (optional).
-        :returns: A list of dicts which contain information about the object.
+        :returns: A list of dictionaries which contain information about the
+            object.
 
         Example::
 
@@ -362,6 +369,7 @@ class BulkApiMixin(object):
             future operations.
 
         Example::
+
             >>> client.new_shim_job('fD9EDV2YROG7pSkBa-hLPg')
             {u'audit_log': [[u'2013-05-16T12:06:08',
                              u'triangle3.local',
@@ -421,7 +429,7 @@ class BulkApiMixin(object):
         :param job_id: Job identifier.
         :param data: A dictionary which can contain the following keys:
             * `status`: New status of the job.
-            * `msg`: A status messages.
+            * `msg`: A status message.
 
         Example::
 
