@@ -2,19 +2,34 @@
 
 import copy
 import json
+import math
 import os
+import sys
 from ConfigParser import SafeConfigParser
 from numbers import Number
 
-from squirro.common.dependency import DependencyNotFound, get_injected
-
 from .base import SquirroClient
 from .exceptions import NotFoundError
 
+# Some magic for providing test support for the Squirro Client. squirro.common
+# is not available on customer's machines, so we don't use get_injected there.
+try:
+    from squirro.common.dependency import DependencyNotFound, get_injected
+
+    def get_client_cls():
+        try:
+            return get_injected('squirro_client_cls')
+        except DependencyNotFound:
+            return SquirroClient
+
+except ImportError:
+    def get_client_cls():
+        return SquirroClient
+
 
 class ItemUploader(object):
-    """Item uploader class. Defaults are loaded from your home directories
-    .squirrorc.
+    """Item uploader class. Defaults are loaded from the ``.squirrorc`` file in
+    the current user's home directory.
 
     :param token: User refresh token.
     :param project_id: Identifier of the project, optional but one of
@@ -29,11 +44,19 @@ class ItemUploader(object):
     :param cluster: Cluster to connect to. This only needs to be changed
         for on-premise installations.
     :param batch_size: Number of items to send in one request. This should
-        be lower than 100 depending on your setup, defaults to 50.
+        be lower than 100 depending on your setup. If set to -1 the optimal
+        batch size is calculated from the items. Defaults to -1.
+    :param bulk_index: If set to `True` the cluster is instructed to index
+        data in bulk.
+    :param bulk_index_add_batch_identifier: If set to `True` the cluster is
+        instructed to add a batch identifier to each item during bulk
+        indexing.
+    :param bulk_index_add_summary_from_body: If set to `True` the cluster is
+        instructed to add the summary from the body during bulk indexing.
     :param config_file: Configuration file to use, defaults to
         ~/.squirrorc
     :param config_section: Section of the .ini file to use, defaults to
-        squirro
+        ``squirro``.
     :param processing_config: A dictionary which contains specific instructions
         which are used while processing items for the source.
     :param source_id: Source which should be used. If passed in together with
@@ -61,7 +84,7 @@ class ItemUploader(object):
     Configuration:
 
     The :class:`ItemUploader` can load its settings from a configuration file.
-    The default section is ``squirro`` and may be overriden by the parameter
+    The default section is ``squirro`` and may be overridden by the parameter
     ``config_section`` to allow for multiple sources/projects.
 
     Example configuration::
@@ -78,7 +101,7 @@ class ItemUploader(object):
                  config_file=None, config_section=None,
                  processing_config=None, steps_config=None,
                  source_id=None, source_secret=None,
-                 **kwargs):
+                 timeout_secs=None, **kwargs):
 
         self.config = None
         kwargs.update(locals())
@@ -86,7 +109,13 @@ class ItemUploader(object):
         self._load_config(kwargs)
         self._validate_config()
         self.client = None
-        self._create_squirro_client(client_cls)
+
+        # include 500 in the status codes for retry handling, this makes the
+        # actual data loading more robust for transient failures (e.g. topic
+        # proxy outages) which are reported as 500
+        self._create_squirro_client(
+            client_cls, retry_status_forcelist=frozenset([500, 502, 503, 504]),
+            timeout_secs=timeout_secs)
 
         self.project_id = None
         self.object_id = None
@@ -160,22 +189,20 @@ class ItemUploader(object):
             raise ValueError('Parameters "project_id" and "project_title" are '
                              'mutually exclusive')
 
-    def _create_squirro_client(self, client_cls):
+    def _create_squirro_client(self, client_cls, **kwargs):
         """Creates a Squirro client"""
-        kwargs = {}
         if all([self.config.get('topic_api_url'),
                 self.config.get('user_api_url'),
                 self.config.get('provider_api_url')]):
-            kwargs = {'topic_api_url': self.config['topic_api_url'],
-                      'user_api_url': self.config['user_api_url']}
+            kwargs.update({
+                'topic_api_url': self.config['topic_api_url'],
+                'user_api_url': self.config['user_api_url']
+            })
         else:
-            kwargs = {'cluster': self.config['cluster']}
+            kwargs['cluster'] = self.config['cluster']
 
         if not client_cls:
-            try:
-                client_cls = get_injected('squirro_client_cls')
-            except DependencyNotFound:
-                client_cls = SquirroClient
+            client_cls = get_client_cls()
 
         self.client = client_cls(None, None, **kwargs)
         self.client.authenticate(refresh_token=self.config['token'])
@@ -221,6 +248,25 @@ class ItemUploader(object):
             }
         }
 
+        # check if bulk indexing is requested
+        bulk_index = self.config.get('bulk_index')
+        if bulk_index:
+            source_config['bulk_index'] = bulk_index
+
+        # bulk indexing: add batch identifier
+        bulk_index_add_batch_identifier = self.config.get(
+            'bulk_index_add_batch_identifier')
+        if bulk_index_add_batch_identifier:
+            source_config['bulk_index_add_batch_identifier'] = \
+                bulk_index_add_batch_identifier
+
+        # bulk indexing: add summary from body
+        bulk_index_add_summary_from_body = self.config.get(
+            'bulk_index_add_summary_from_body')
+        if bulk_index_add_summary_from_body:
+            source_config['bulk_index_add_summary_from_body'] = \
+                bulk_index_add_summary_from_body
+
         if processing_config is not None:
             assert isinstance(processing_config, dict), 'dict required'
             source_config['processing'].update(processing_config)
@@ -253,7 +299,7 @@ class ItemUploader(object):
 
         headers = {'Content-Type': 'application/json'}
 
-        batch_size = self.config['batch_size']
+        batch_size = self._get_batch_size(items)
         for idx in xrange(0, len(items), batch_size):
             for item in items[idx:idx + batch_size]:
                 self._fixup_keywords(item)
@@ -263,6 +309,27 @@ class ItemUploader(object):
                                                headers=headers)
             res.raise_for_status()
 
+    def _get_batch_size(self, items):
+        """Gets the batch size from config or calculates the optimal size based
+        on the items if batch_size is set to -1.
+        """
+        MAX_BYTE_SIZE = 5e+7  # 50MB, allows for some json dumps overhead and
+        # ES bulk action instructions
+
+        batch_size = self.config['batch_size']
+
+        if batch_size == -1:
+            items_bytes_size = sys.getsizeof(items)
+            if items_bytes_size < MAX_BYTE_SIZE:
+                batch_size = len(items)
+            else:
+                # how many chunks do we split the items into?
+                nof_chunks = math.ceil(float(items_bytes_size) / MAX_BYTE_SIZE)
+                assert nof_chunks >= 1
+                batch_size = min(1, math.floor(float(len(items)) / nof_chunks))
+
+        return int(batch_size)
+
     def _fixup_keywords(self, item):
         """Validates and fixes keywords of `item`"""
         if item.get('keywords') is None:
@@ -285,7 +352,10 @@ class ItemUploader(object):
         'source_name': 'Upload',
         'object_id': 'default',
         'cluster': 'https://next.squirro.net',
-        'batch_size': 50,
+        'batch_size': -1,  # auto calculate
+        'bulk_index': False,
+        'bulk_index_add_batch_identifier': False,
+        'bulk_index_add_summary_from_body': False,
     }
 
     # options to be converted to int
@@ -297,5 +367,6 @@ class ItemUploader(object):
     CONFIG_KEYS = [
         'token', 'cluster', 'project_id', 'project_title',
         'object_id', 'source_name', 'source_ext_id', 'batch_size',
-        'user_api_url', 'topic_api_url', 'provider_api_url'
+        'user_api_url', 'topic_api_url', 'provider_api_url', 'bulk_index',
+        'bulk_index_add_batch_identifier', 'bulk_index_add_summary_from_body',
     ]
