@@ -5,6 +5,11 @@ import json
 class UserApiMixin(object):
     """Mixin class which encapsulates the functionality of the user API
     service.
+
+    All the methods of this class are made available in the
+    [[SquirroClient]] class.
+
+    {TOC}
     """
 
     def get_user_data(self, user_id, api_version='v1'):
@@ -294,7 +299,7 @@ class UserApiMixin(object):
 
         :param user_id: Identifier of the user to delete.
 
-        Example:
+        Example::
 
             >>> client.delete_user('H5Qv-WhgSBGW0WL8xolSCQ')
         """
@@ -310,7 +315,7 @@ class UserApiMixin(object):
 
         :param project_id: Project identifier.
 
-        Example:
+        Example::
 
             >>> client.get_project_members('2sic33jZTi-ifflvQAVcfw')
             {u'members': [{
@@ -355,7 +360,7 @@ class UserApiMixin(object):
         :param user_id: Identifier of the user to be added (can't be combined with `group_id`).
         :param group_id: Identifier of the group to be added (can't be combined with `user_id`).
 
-        Example:
+        Example::
 
             >>> client.add_project_member('2sic33jZTi-ifflvQAVcfw', 'member',
                                           user_id='H5Qv-WhgSBGW0WL8xolSCQ')
@@ -401,7 +406,7 @@ class UserApiMixin(object):
         :param member_id: Identifier of the user or group to be updated.
         :param role: New role of the member. Allowed values: `reader`, `member`, `admin`.
 
-        Example:
+        Example::
 
             >>> client.update_project_member('2sic33jZTi-ifflvQAVcfw', 'H5Qv-WhgSBGW0WL8xolSCQ', 'admin')
             {
@@ -426,7 +431,7 @@ class UserApiMixin(object):
         :param project_id: Project identifier of the project to remove the member from.
         :param member_id: Identifier of the user or group to be removed.
 
-        Example:
+        Example::
 
             >>> client.delete_project_member('2sic33jZTi-ifflvQAVcfw',
                                              'H5Qv-WhgSBGW0WL8xolSCQ')
@@ -442,7 +447,7 @@ class UserApiMixin(object):
     def get_groups(self):
         """Returns all groups within the logged-in users tenant.
 
-        Example:
+        Example::
 
             >>> client.get_groups()
             [{u'id': u'1jo-WmmyRvC8UYtPMIQaoQ',
@@ -471,7 +476,7 @@ class UserApiMixin(object):
 
         :param name: Group name
 
-        Example:
+        Example::
 
             >>> client.create_group('Team East')
             {
@@ -492,7 +497,7 @@ class UserApiMixin(object):
 
         :param group_id: Identifier of the group for which the details are returned.
 
-        Example:
+        Example::
 
             >>> client.get_group('1jo-WmmyRvC8UYtPMIQaoQ')
             {
@@ -522,7 +527,7 @@ class UserApiMixin(object):
         :param group_id: Identifier of the group to modify.
         :param name: New name of the group.
 
-        Example:
+        Example::
 
             >>> client.update_group('1jo-WmmyRvC8UYtPMIQaoQ', 'Team West')
             {
@@ -551,7 +556,7 @@ class UserApiMixin(object):
 
         :param group_id: Identifier of the group to delete.
 
-        Example:
+        Example::
 
             >>> client.delete_group('1jo-WmmyRvC8UYtPMIQaoQ')
         """
@@ -567,7 +572,7 @@ class UserApiMixin(object):
         :param group_id: Identifier of the group to add the user to.
         :param user_id: Identifier of the user to add to the group.
 
-        Example:
+        Example::
 
             >>> client.add_group_member('1jo-WmmyRvC8UYtPMIQaoQ',
                                         'H5Qv-WhgSBGW0WL8xolSCQ')
@@ -592,7 +597,7 @@ class UserApiMixin(object):
         :param group_id: Identifier of the group from which to remove the user.
         :param user_id: Identifier of the user to remove from the group.
 
-        Example:
+        Example::
 
             >>> client.delete_group_member('1jo-WmmyRvC8UYtPMIQaoQ',
                                            'H5Qv-WhgSBGW0WL8xolSCQ')
